#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <X11/XF86keysym.h>
#include "support.h"
#include "interface.h"
#include "main.h"
#include "db.h"
#include "calllist.h"
#include "utils.h"

GtkWidget *calllist_widget;
GtkWidget *notebook;


#ifndef GDK_XP_Phone
#define GDK_XP_Phone XF86XK_Phone
#endif

enum {
	COL_TIME = 0,
	COL_NUMBER,
	COL_DURATION,
	NUM_COLS
};

static gboolean
calllist_key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data)
{
	gint current_page,max_page,offset;
	GtkWidget *scrolled,*page;
	GtkTreeSelection *selection;
	GtkTreeIter iter;
	GtkTreeModel *model=NULL;
	GList *children;
	GValue value;

	if (event->type != GDK_KEY_PRESS)
		return FALSE;
	switch(event->keyval) {
	case GDK_XP_Phone:
		printf("at 1\n");
		current_page=gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
		printf("at 2\n");
		scrolled=gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), current_page);
		printf("at 3\n");
		children=gtk_container_get_children(GTK_CONTAINER(scrolled));
		printf("at 4\n");
		page=GTK_WIDGET(children->data);
		printf("at 5\n");
		printf("at 6\n");
		selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(page));
		printf("at 7 selection=%p\n",selection);
		gtk_tree_selection_get_selected(selection, &model, &iter);
		printf("at 8 model=%p iter=%d,%p,%p,%p\n",model, iter.stamp, iter.user_data,iter.user_data2,iter.user_data3);
		printf("GTK_IS_TREEMODEL=%d\n", GTK_IS_TREE_MODEL(model));
		{
			GtkTreeModelIface *iface;
			iface = GTK_TREE_MODEL_GET_IFACE (model);
			printf("iface=%p\n", iface);
			printf("iface->get_value=%p\n", iface->get_value);

		}
		gtk_tree_model_get_value(model, &iter, COL_NUMBER, &value);
		printf("at 9\n");
		prepare_dial(g_value_get_string(&value));
		printf("at 10\n");
		g_list_free(children);
        	return TRUE;
	case GDK_Left:
	case GDK_Right:
		current_page=gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
		max_page=gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook));
		offset=event->keyval == GDK_Right ? 1 : -1;
		current_page+=offset;
		if (current_page >= max_page)
			current_page=0;
		if (current_page < 0)
			current_page=max_page-1;
		gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook),current_page);
        	return TRUE;

	}
	return FALSE;
}

static GtkTreeModel *create_and_fill_model(int list)
{
	GtkListStore *liststore;
	GtkTreeIter toplevel;
	GSList *res, *iter;
	char *duration,*time_str;
	time_t now=time(NULL);


	liststore = gtk_list_store_new(NUM_COLS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
	printf("exec db_call_query\n");
	res=db_call_query(list);
	printf("exec db_call_query res=%p\n",res);

	
	for (iter = res; iter; iter = iter->next)
	{
		struct call_entry *e = iter->data;
		gtk_list_store_append(liststore, &toplevel);
		time_str=format_date(&now, &e->time);
		duration=format_duration(e->duration);
		gtk_list_store_set(liststore, &toplevel, COL_TIME, time_str, COL_NUMBER, e->number, COL_DURATION, duration,  -1);
		g_free(duration);
		g_free(time_str);
		db_call_entry_free(e);
	}
	g_slist_free(res);

		
	/* Append a top level row and leave it empty */

	printf("return\n");
	return GTK_TREE_MODEL(liststore);
}

static void create_view_and_model(GtkWidget *view, int list, int new)
{
	GtkTreeModel *model;
	GtkTreePath *path;

	printf("create_view_and_model\n");
	if (new) {
		gtk_tree_view_column_new_text(view, "Time", COL_TIME);
		gtk_tree_view_column_new_text(view, "Number", COL_NUMBER);
		gtk_tree_view_column_new_text(view, "Duration", COL_DURATION);
	}

	model = create_and_fill_model(list);

	gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);

	g_object_unref(model);	/* destroy model automatically with view */


	gtk_tree_selection_set_mode(gtk_tree_view_get_selection (GTK_TREE_VIEW(view)), GTK_SELECTION_SINGLE);
	path=gtk_tree_path_new_from_indices(0,-1);
	gtk_tree_view_set_cursor(GTK_TREE_VIEW(view), path, NULL, FALSE);		
}

void calllist_unrealize(GtkWidget *widget, gpointer user_data)
{
	calllist_widget=NULL;
}

void calllist_open(enum calllist_page tab)
{
	GtkWidget *outgoing, *incoming, *missed;
	int new=0;

	printf("calllist_open\n");
	if (!calllist_widget) {
		calllist_widget = create_Call_List();
		g_signal_connect ((gpointer) calllist_widget, "key-press-event", (GCallback) calllist_key_pressed, NULL);
		g_signal_connect ((gpointer) calllist_widget, "unrealize", (GCallback) calllist_unrealize, NULL);
		new=1;
	} else
		window_raise(calllist_widget);

	outgoing = lookup_widget(calllist_widget, "outgoing_list");
	incoming = lookup_widget(calllist_widget, "incoming_list");
	missed = lookup_widget(calllist_widget, "missed_list");
	notebook = lookup_widget(calllist_widget, "SMSList_notebook");
	create_view_and_model(outgoing,CALL_LIST_OUTGOING, new);
	create_view_and_model(incoming,CALL_LIST_INCOMING, new);
	create_view_and_model(missed,CALL_LIST_MISSED_REJECTED, new);
	gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), (gint) tab);
	gtk_widget_grab_focus(outgoing);	
	gtk_widget_show(calllist_widget);

}
