#define DBUS_API_SUBJECT_TO_CHANGE
#include <stdlib.h>
#include <libintl.h>
#include <dbus/dbus.h>
#include <dbus/dbus-glib.h>


#include "config.h"
#define _(x) gettext(x)


static DBusConnection *connection;

#define PHONE_SERVICE_NAME "de.linuxdevelopment.gomunicator"
#define PHONE_INTERFACE_NAME PHONE_SERVICE_NAME

#define WRONG_ARGS_ERROR "de.linuxdevelopment.gomunicator.Error.WrongArgs"

DBusHandlerResult dial_handle_dbus_request (DBusConnection *connection, DBusMessage *message)
{
	DBusMessageIter iter;
	DBusMessage *reply;
	char *number;

	printf("dial_handle_dbus_request\n");
	dbus_message_iter_init (message, &iter);
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING)
		goto wrong_args;
#ifdef HAVE_DBUS_MESSAGE_ITER_GET_BASIC
	dbus_message_iter_get_basic (&iter, &number);
#else
	number = dbus_message_iter_get_string (&iter);
#endif
	printf("number %s\n", number);
	prepare_dial(number);
	return DBUS_HANDLER_RESULT_HANDLED;

	wrong_args:
	reply = dbus_message_new_error (message, WRONG_ARGS_ERROR, "Wrong arguments for Dial method");
	dbus_connection_send (connection, reply, NULL);
	dbus_message_unref (reply);
	return DBUS_HANDLER_RESULT_HANDLED;
}

static DBusHandlerResult phone_handler_func(DBusConnection *connection, DBusMessage *message, void *user_data)
{
	printf("phone_handler_func\n");
	if (dbus_message_is_method_call (message, PHONE_INTERFACE_NAME, "Dial"))
		return dial_handle_dbus_request (connection, message);

  	if (dbus_message_is_signal (message,
#ifdef DBUS_INTERFACE_LOCAL
		DBUS_INTERFACE_LOCAL,
#else
		DBUS_INTERFACE_ORG_FREEDESKTOP_LOCAL,
#endif
		"Disconnected"))
		exit (0);
	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
}

static DBusObjectPathVTable dbus_phone_vtable = {
	NULL,
	phone_handler_func,
	NULL
};

void dbus_init(void)
{
	static const char *object_path = "/de/linuxdevelopment/gomunicator";
	DBusError error;

	dbus_error_init(&error);

	connection = dbus_bus_get(DBUS_BUS_SESSION, &error);
	if (connection) {
		dbus_connection_setup_with_g_main(connection, NULL);
		dbus_connection_register_object_path (connection, object_path, &dbus_phone_vtable, NULL);
#ifdef HAVE_DBUS_MESSAGE_ITER_GET_BASIC
		dbus_bus_request_name(connection, PHONE_SERVICE_NAME, 0, &error);
#else
		dbus_bus_acquire_service(connection, PHONE_SERVICE_NAME, 0, &error);
#endif
		if (dbus_error_is_set(&error)) {
#ifdef HAVE_DBUS_MESSAGE_ITER_GET_BASIC
			gpe_error_box_fmt(_("Failed to request name: %s"), error.message);
#else
			gpe_error_box_fmt(_("Failed to acquire service: %s"), error.message);
#endif
			dbus_error_free (&error);
		}
	} else {
		gpe_error_box_fmt (_("Failed to open connection to session message bus: %s"), error.message);
		dbus_error_free (&error);
	}
}
