/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gdk/gdkx.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <X11/XF86keysym.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>

#include "main.h"
#include "interface.h"
#include "support.h"

#include "phoneComms.h"
#include "sound.h"
#include "db.h"
#include "dbus.h"
#include "io.h"
#include "utils.h"
#include "calllist.h"
#include "configmodel.h"
#include "mixer.h"

#define PID_FILENAME "/tmp/gomunicator.pid"

GtkWidget *MainWindow, *TimeLabel, *SignalImage, *BatteryImage;
GtkWidget *DialAnswerButton, *HangupIgnoreButton, *StatusLabel;
GtkWidget *MissedLabel, *MsgsLabel, *KeypadTable, *OperatorEntry, *DialEntry, *NameEntry;
GtkWidget *BackspaceButton, *Entry2, *Config;

time_t LastUpdateTime;
int MissedCount = 0;
char imageFilename[30 + PIXMAP_DIR_LEN] = PIXMAP_DIR;
int profile, i;

int callid=-1;

int dialanswer_mode;

void
audio_off(void)
{
	mixer_phone_volume_set(0);
	mixer_mic_power_set(0);
}

void
audio_on(void)
{
	mixer_phone_volume_set(33);
	mixer_mic_power_set(1);
}

void set_buttons(void)
{
	int sensitive=1;

	switch (phoneState) {
	case psPIN:
		gtk_label_set_text(GTK_LABEL(StatusLabel), "PIN");
		sensitive=*(gtk_entry_get_text(GTK_ENTRY(DialEntry))) != 0;
		gtk_button_set_label(GTK_BUTTON(DialAnswerButton), "Enter");
		gtk_button_set_label(GTK_BUTTON(HangupIgnoreButton), "Cancel");
		gtk_widget_set_sensitive(HangupIgnoreButton, TRUE);
		break;
	case psIdle:
		gtk_label_set_text(GTK_LABEL(StatusLabel), "Idle");
		gtk_widget_set_sensitive(HangupIgnoreButton, FALSE);
		gtk_widget_set_sensitive(KeypadTable, TRUE);
		gtk_entry_set_editable(GTK_ENTRY(DialEntry), TRUE);
		if (*(gtk_entry_get_text(GTK_ENTRY(DialEntry))) != 0) {
			dialanswer_mode++;
			gtk_button_set_label(GTK_BUTTON(DialAnswerButton), "Dial");
		} else
			gtk_button_set_label(GTK_BUTTON(DialAnswerButton), "Last");
		break;
	case psRinging:
		gtk_label_set_text(GTK_LABEL(StatusLabel), "Ringing");
		gtk_button_set_label(GTK_BUTTON(DialAnswerButton), "Answer");
		gtk_button_set_label(GTK_BUTTON(HangupIgnoreButton), "Ignore");
		gtk_widget_set_sensitive(HangupIgnoreButton, TRUE);
		gtk_widget_set_sensitive(KeypadTable, FALSE);
		gtk_entry_set_editable(GTK_ENTRY(DialEntry), FALSE);
		break;
	case psOnline:
		sensitive=0;
		gtk_label_set_text(GTK_LABEL(StatusLabel), "Online");
		gtk_button_set_label(GTK_BUTTON(HangupIgnoreButton), "End call");
		gtk_widget_set_sensitive(HangupIgnoreButton, TRUE);
		gtk_widget_set_sensitive(KeypadTable, TRUE);
		break;
	}
	if (sensitive)
		gtk_widget_set_sensitive(DialAnswerButton, TRUE);
	else
		gtk_widget_set_sensitive(DialAnswerButton, FALSE);

}

void dialanswer_button_pressed(void)
{
	const gchar *dialEntryText;
	int entryIndex, stringIndex;
	char dialChar, dialString[129] = "ATD";

	switch (phoneState) {
	case psPIN:
		dialEntryText = gtk_entry_get_text(GTK_ENTRY(DialEntry));
		gtk_entry_set_visibility(GTK_ENTRY(DialEntry), TRUE);
		sendPIN(dialEntryText);
		if (phoneConfig.moduleType == M_HIMALAYA)
		{
			setCOPS(0);
		}
		checkPIN();
		checkCOPS();
		checkSetup();
		checkCOPS();
		break;
	case psIdle:
		dialEntryText = gtk_entry_get_text(GTK_ENTRY(DialEntry));
		if (! strlen(dialEntryText)) {
			calllist_open(PAGE_OUTGOING);
			return;
		}
		for (entryIndex = 0, stringIndex = 3;
		     dialEntryText[entryIndex] != 0 && stringIndex < 127;
		     ++entryIndex) {
			dialChar = dialEntryText[entryIndex];
			if ((dialChar >= '0' && dialChar <= '9') || dialChar == ','
			    || dialChar == '!' || dialChar == '*'
			    || dialChar == '#' || dialChar == '@' || dialChar == '+')
				dialString[stringIndex++] = dialChar;
			else {
				dialChar &= ~('a' - 'A');
				if (dialChar < 'S')
					dialString[stringIndex++] =
					    (dialChar - 'A') / 3 + '2';
				else if (dialChar < 'Z')
					dialString[stringIndex++] =
					    (dialChar - 'Q') / 3 + '7';
				else if (dialChar == 'Z')
					dialString[stringIndex++] = '9';
			}		//if ((dialChar >= '0' && dialChar <= '9') || ...dialChar =='@')
		}			//for (entryIndex = ... 0; dialEntryText[...] != 0 ...; ++entryIndex)

		if (stringIndex == 3)
			return;
		dialString[stringIndex] = '\0';
		display_lookup_set_name(dialString+3);	
		callid=db_call_new(dialString+3,CALL_LIST_OUTGOING);
		dialString[stringIndex] = ';';
		dialString[stringIndex + 1] = 0;
		if (makeCall(dialString)) {
			set_buttons();
			dialString[stringIndex] = 0;
			DialedOut(dialString + 3);
		}			//if (phoneGetReply(dialString))
		break;
	case psRinging:
		stopSound();
		answerPhone();
		if (phoneState == psOnline)
			StartCall();
		break;
	case psOnline:
		break;
	}
}

void hangupignore_button_pressed(void)
{
	switch (phoneState) {
	case psPIN:
		break;
	case psIdle:
		break;
	case psRinging:
		StopRinging(FALSE);
		break;
	case psOnline:
		EndCall();
	}
}

void StartRinging(void)
{
	startSound(RING_SAMPLE_FILENAME);
	vibra_on();
	set_buttons();
}				//StartRinging


void StopRinging(gboolean missed)
{
	char labelText[6];

	set_buttons();
	hangupPhone();
	vibra_off();
	stopSound();
	if (missed) {
		snprintf(labelText, 10, "%d", ++MissedCount);
		gtk_label_set_text(GTK_LABEL(MissedLabel), labelText);
	}			//if (missed)
}				//StopRinging


void StartCall(void)
{
	set_buttons();
	vibra_off();
	stopSound();
	gtk_label_set_text(GTK_LABEL(TimeLabel), "0:00");
	gtk_widget_set_sensitive(TimeLabel, TRUE);
}				//StartCall


void DialedOut(char *dialString)
{
	int stringLength, statusIndex = 7;
	char statusString[32] = "Dialed\n";

	if ((stringLength = strlen(dialString)) > 20)
		stringLength = 20;
	while (stringLength > 7) {
		strncpy(statusString + statusIndex, dialString, 4);
		statusString[statusIndex + 4] = '\n';
		statusIndex += 5;
		dialString += 4;
		stringLength -= 4;
	}			//while (stringLength > 7)
	if (stringLength > 4) {
		strncpy(statusString + statusIndex, dialString,
			stringLength / 2);
		statusString[statusIndex + stringLength / 2] = '\n';
		statusIndex += stringLength / 2 + 1;
		dialString += stringLength / 2;
		stringLength -= stringLength / 2;
	}			//if (stringLength > 4)
	strncpy(statusString + statusIndex, dialString, stringLength);
	statusString[statusIndex + stringLength] = 0;
	gtk_label_set_text(GTK_LABEL(StatusLabel), statusString);
	set_buttons();
	gtk_label_set_text(GTK_LABEL(TimeLabel), "0:00");
	gtk_widget_set_sensitive(TimeLabel, TRUE);
//  gtk_widget_set_visible(TimeLabel, TRUE);
}				//DialedOut


void EndCall(void)
{
	hangupPhone();
	set_buttons();
	gtk_widget_set_sensitive(TimeLabel, FALSE);
	gtk_entry_set_editable(GTK_ENTRY(DialEntry), TRUE);
}				//EndCall


void UpdateTime(int seconds)
{
	char timeString[10];

	snprintf(timeString, 10, "%d:%02d", seconds / 60, seconds % 60);
	gtk_label_set_text(GTK_LABEL(TimeLabel), timeString);
}				//UpdateTime


void UpdateBattery(int level, gboolean charging)
{
	if (level < 0)
		strcpy(imageFilename + PIXMAP_DIR_LEN,
		       "BatteryUnknown.xpm");
	else if (charging)
		sprintf(imageFilename + PIXMAP_DIR_LEN,
			"BatteryCharge%d.xpm", level);
	else
		sprintf(imageFilename + PIXMAP_DIR_LEN, "Battery%d.xpm",
			level);
	printf("Battery image:%s\n", imageFilename);
	gtk_image_set_from_file(GTK_IMAGE(BatteryImage), imageFilename);
}				// UpdateBattery


void UpdateSignal(int level)
{
	if (level > 5)
		strcpy(imageFilename + PIXMAP_DIR_LEN, "Signal0.xpm");
	else

		sprintf(imageFilename + PIXMAP_DIR_LEN, "Signal%d.xpm",
			level);
	printf("Signal image:%s\n", imageFilename);
	gtk_image_set_from_file(GTK_IMAGE(SignalImage), imageFilename);
}				//UpdateSignal


void NewSMS(int totalMsgs)
{
	char counterString[3] = "C+";

	if (totalMsgs < 101)
		switch (totalMsgs) {
		case 1:
			strcpy(counterString, "I");
			break;

		case 2:
			strcpy(counterString, "II");
			break;

		case 4:
			strcpy(counterString, "IV");
			break;

		case 5:
			strcpy(counterString, "V");
			break;

		case 6:
			strcpy(counterString, "VI");
			break;

		case 9:
			strcpy(counterString, "IX");
			break;

		case 10:
			strcpy(counterString, "X");
			break;

		case 11:
			strcpy(counterString, "XI");
			break;

		case 15:
			strcpy(counterString, "XV");
			break;

		case 20:
			strcpy(counterString, "XX");
			break;

		case 40:
			strcpy(counterString, "XL");
			break;

		case 50:
			strcpy(counterString, "L");
			break;

		case 51:
			strcpy(counterString, "LI");
			break;

		case 55:
			strcpy(counterString, "LV");
			break;

		case 60:
			strcpy(counterString, "LX");
			break;

		case 90:
			strcpy(counterString, "XC");
			break;

		case 100:
			strcpy(counterString, "C");
			break;

		default:
			snprintf(counterString, 3, "%d", totalMsgs);
			break;
		}		//if (totalMsgs < 101) switch (totalMsgs)
	gtk_label_set_text(GTK_LABEL(MsgsLabel), counterString);
}				//NewSMS

void display_set_operator(const char *text)
{
	int position=0;
	
        gtk_editable_delete_text(GTK_EDITABLE(OperatorEntry), 0, -1);
	if (text)
		gtk_editable_insert_text(GTK_EDITABLE(OperatorEntry), text, strlen(text), &position);
        gtk_editable_set_position(GTK_EDITABLE(OperatorEntry), -1);

}


void display_set_number(const char *text)
{
	int position=0;
	
        gtk_editable_delete_text(GTK_EDITABLE(DialEntry), 0, -1);
	if (text)
		gtk_editable_insert_text(GTK_EDITABLE(DialEntry), text, strlen(text), &position);
        gtk_editable_set_position(GTK_EDITABLE(DialEntry), -1);

}

void display_set_name(const char *text)
{
	int position=0;
	
        gtk_editable_delete_text(GTK_EDITABLE(NameEntry), 0, -1);
	if (text)
		gtk_editable_insert_text(GTK_EDITABLE(NameEntry), text, strlen(text), &position);
        gtk_editable_set_position(GTK_EDITABLE(NameEntry), -1);
}

void display_lookup_set_name(const char *number)
{
	display_set_name(db_lookup_number(number));
}

int checkFUN(void)
{
	int fun=getFUN();
	if (fun != 1) {
		printf("Enabling Phone\n");
		sendFUN(1);
	}
	return 0;
}

int checkCOPS(void)
{
	int cops;
	if (phoneState != psPIN) {
		cops=getCOPS();
		printf("cops=%d\n", cops);
		if (! cops)
			setCOPS(0);
	
	}
	return 0;
}

int checkPIN(void)
{
	GtkWidget *dialog;

	if (!phoneGetSecurity()) {
		dialog = gtk_message_dialog_new(GTK_WINDOW(MainWindow),
						GTK_DIALOG_DESTROY_WITH_PARENT,
						GTK_MESSAGE_ERROR,
						GTK_BUTTONS_CLOSE,
						"Could not get security status\n"
						"Is SIM card missing?");
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
		return 2;
	}			//if (!phoneGetReply("AT+CPIN?"))


	if (phoneSecurity != psReady) {
		phoneState = psPIN;
		display_set_operator("Enter PIN:");
		set_buttons();
		gtk_entry_set_visibility(GTK_ENTRY(DialEntry), FALSE);
	} else {
		display_set_number(NULL);
		display_set_name(NULL);
		phoneState = psIdle;
		set_buttons();
	}

	return 0;
}				//checkPIN

int initPhone(void)
{
	int 		returnValue=0;
	GtkWidget 	*dialog;
	
	if (!openPhone()) {
		dialog = gtk_message_dialog_new(GTK_WINDOW(MainWindow),
					   GTK_DIALOG_DESTROY_WITH_PARENT,
					   GTK_MESSAGE_INFO,
					   GTK_BUTTONS_CLOSE,
					   "Cannot find a phone at %s ",
					   phoneConfig.devName);
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
		return 1;
	}			//if (!openPhone())

	printf("Sending initialisation string to phone\n");
	sendInitString();
	if (phoneConfig.moduleType == M_HIMALAYA)
	{
		int i;
		sleep(4);
		for (i = 0; i < 5; i++)
			ReadPhoneStartGarbage(1);
	}
	if ((phoneConfig.moduleType != M_HIMALAYA) &&
		/* the first command is always in "echo" mode
		 * so sending it twice makes the code in phoneComm.c
		 * sort its state machine out
		 */
		(!phoneGetOK("ATV1E0", TRUE)) && //is phone already in AT mode?
			((phoneGetOK("ATE0", TRUE) == FALSE) && // why twice?
			  (phoneGetOK("ATE0", TRUE) == FALSE))) {

		dialog = gtk_message_dialog_new(GTK_WINDOW
					   (MainWindow),
					   GTK_DIALOG_DESTROY_WITH_PARENT,
					   GTK_MESSAGE_ERROR,
					   GTK_BUTTONS_CLOSE,
					   "Could not initialise the phone at %s via string %s",
					   phoneConfig.devName,
					   phoneConfig.initATCmd);
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
		return 2;
	}

	tryReadPhone(2);
	if (phoneConfig.moduleType != M_HIMALAYA)
	{
		checkFUN();
	}
	else
	{
		phoneGetOK("", TRUE);
		sendSetEquipmentState(1);
	}
	checkPIN();
	checkCOPS();
	if (phoneConfig.moduleType == M_HIMALAYA)
	{
		phoneGetOK("", TRUE);
	}

	printf("init_phone returning %d\n", returnValue);
	return returnValue;
}				//initPhone


void suspendHandler(int signalNumber)
{
	int suspendFile;

	printf("gomunicator suspending.\n");
	closePhone();
	if ((suspendFile = open(SUSPEND_FILENAME, O_RDONLY | O_CREAT,
				S_IRWXU | S_IRWXG | S_IRWXO)) > 0) {
		write(suspendFile, &suspendFile, 1);	//just so the file has some content
		close(suspendFile);
	}			//if ((suspendFile = open(SUSPEND_FILENAME, ...)) > 0)
}				//suspendHandler


void resumeHandler(int signalNumber)
{
	printf("gomunicator resuming.\n");
	openPhone();
	unlink(SUSPEND_FILENAME);
}				//resumeHandler


int setupPhone(int function)
{
	switch (function) {
	case 1:
		printf("Setting \"SMS message storage\" %s\n", phoneConfig.smsStoreATCmd);
		phoneGetOK(phoneConfig.smsStoreATCmd, TRUE);
		break;
	case 2:
		printf("Setting SMS mode to PDU (AT+CMGF=0)\n");
		phoneGetOK("AT+CMGF=0", TRUE);
		break;
	case 3:
		printf("Setting notifications (AT+CNMI=2,1,0,0,0)\n");
		phoneGetOK("AT+CNMI=2,1,0,0,0", TRUE);
		break;
	case 4:
		printf("Deactivate cell broadcast reception (AT+CSCB=1)\n");
		phoneGetOK("AT+CSCB=1", TRUE);
		break;
	case 5:
		phoneGetOK("AT+CLIP=1", TRUE);
		phoneGetOK("AT+CMGL=0", TRUE);
		phoneGetOK("ATH0", TRUE);
		break;
	}
	return 0;
}

int checkSetup(void)
{
	static int setup_done;

	if (! setup_done && phoneState == psIdle) {
		for (i = 1; i < 6; i++) {
                        setupPhone(i);
                }
		setup_done=1;
	}

	return 0;
}

void usage(char *option)
{

	fprintf(stderr,
		"gomunicator (%s) usage:\n\t gomunicator [Options....\n\n]"
		"Where options are;\n" "  -V  --Version  print version\n"
		" -h, --help display this help file \n"
		" -v, --verbose  enable verbose console\n"
		" -d, --device <PATH> specify phone device\n"
		" -i, --init <STRING> specify init string\n"
		" -p  --profile <0|1|2> specfiy the device profile\n",
		option);
	unlink(PID_FILENAME);
	exit(1);
}

void profile_use(int option)
{
	switch (option) {
		case 0:
			printf("Using profil 0\n");
			phoneConfig.initATCmd	= "";
			break;
		case 1:
			printf("Using profil 1\n");
			phoneConfig.initATCmd	= "0xF9, 0xF9, 0xF9, 0xF9, 0xF9, 0x03, 0xEF, 0x05, 0xC3, 0x01, 0xF2, 0xF9";
			break;
		case 2:
			printf("Using profil 2\n");
			break;
	}
}

void
newSMS(char *text)
{
	GtkWidget *dialog, *label;
   
	dialog = gtk_dialog_new_with_buttons ("New SMS",
                                         GTK_WINDOW(MainWindow),
                                         GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_STOCK_OK,
                                         GTK_RESPONSE_NONE,
                                         NULL);
	label = gtk_label_new (text);
   
   
	g_signal_connect_swapped (dialog,
			     "response", 
                             G_CALLBACK (gtk_widget_destroy),
                             dialog);

	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), label);
	gtk_widget_show_all (dialog);
}

void showversion()
{
	GtkWidget *dialog;
	dialog = gtk_message_dialog_new(NULL,
					GTK_DIALOG_DESTROY_WITH_PARENT,
					GTK_MESSAGE_INFO,
					GTK_BUTTONS_CLOSE,
					"Version " VERSION);

	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
}

void calldebug()
{
	printf("DEBUG call\n");
}

void mainwindow_raise(void)
{
	window_raise(MainWindow);
}

#ifndef GDK_XP_Phone
#define GDK_XP_Phone XF86XK_Phone
#endif

static gboolean
mainwindow_key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data)
{
	if (event->type != GDK_KEY_PRESS)
		return FALSE;
	switch(event->keyval) {
	case GDK_XP_Phone:
		dialanswer_button_pressed();
        	return TRUE;
	case GDK_Cancel:
		hangupignore_button_pressed();
		return TRUE;
	}
	return FALSE;
}

void
prepare_dial(const char *number)
{
	display_set_number(number);
        display_lookup_set_name(number);
        mainwindow_raise();
}


int main(int argc, char *argv[])
{
	int returnValue, i, pidFile, readLength;
	char pidMessage[128] = "According to the existing file\n"
	    PID_FILENAME
	    ",\ngomunicator is already running with process ID ";
#define pidText (pidMessage + 90)
	GtkWidget *dialog;
	struct sigaction suspendAction, resumeAction;

#ifdef ENABLE_NLS
	bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
	textdomain(GETTEXT_PACKAGE);
#endif

	gtk_set_locale();
	gtk_init(&argc, &argv);
	add_pixmap_directory(PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps/");
	phoneConfig	= read_configuration_options();
	
	returnValue = 0;
	if (access(PID_FILENAME, F_OK) == 0) {
		if ((pidFile = open(PID_FILENAME, O_RDONLY)) < 0)
			returnValue = 6;
		else if ((readLength = read(pidFile, pidText, 15)) < 1)
			returnValue = 7;
		if (returnValue != 0) {
			dialog = gtk_message_dialog_new(NULL,
							GTK_DIALOG_DESTROY_WITH_PARENT,
							GTK_MESSAGE_ERROR,
							GTK_BUTTONS_CLOSE,
							PID_FILENAME
							"\nexists but "
							"could not be read.\nIs gomunicator already running?");
			gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
			return returnValue;
		} else {
			int pid;
			pidText[readLength] = 0;
			pid = atoi(pidText);
			if (!kill(0, pid)) {

				dialog =
				    gtk_message_dialog_new(GTK_WINDOW
							   (MainWindow),
							   GTK_DIALOG_DESTROY_WITH_PARENT,
							   GTK_MESSAGE_ERROR,
							   GTK_BUTTONS_CLOSE,
							   pidMessage);
				gtk_dialog_run(GTK_DIALOG(dialog));
				gtk_widget_destroy(dialog);
				return 8;
			}
		}		//if (returnValue != 0)...else
	}

	if ((pidFile = open(PID_FILENAME,
			    O_WRONLY | O_CREAT | O_TRUNC, 00644)) < 0) {
		dialog = gtk_message_dialog_new(GTK_WINDOW(MainWindow),
						GTK_DIALOG_DESTROY_WITH_PARENT,
						GTK_MESSAGE_ERROR,
						GTK_BUTTONS_CLOSE,
						"Could not create/open\n"
						PID_FILENAME);
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
		return 9;
	}			//if ((pidFile = open("/tmp/gphone.pid", ..., 00644)) < 0)
	snprintf(pidText, 16, "%d\n", getpid());
	write(pidFile, pidText, strlen(pidText));
	close(pidFile);

	for (i = 1; i < argc; i++) {
		if (!strcmp("-h", argv[i]) || !strcmp("--help", argv[i])) {
                        if (++i >= argc)
                                usage(argv[i]);
			else usage(argv[i]);
                        phoneConfig.devName	= argv[i];
                        continue;
                }
		if (!strcmp("-d", argv[i]) || !strcmp("--device", argv[i])) {
			if (++i >= argc)
				usage(argv[i]);
			phoneConfig.devName	= argv[i];
			continue;
		}
		if (!strcmp("-v", argv[i]) || !strcmp("--verbose", argv[i])) {
			printf("verbose\n");
			continue;
		}
		if (!strcmp("-V", argv[i]) || !strcmp("--Version", argv[i])) {
			showversion();
			printf("gomunicator Version \n" VERSION);
			continue;
		}
		if (!strcmp("-p", argv[i]) || !strcmp("--profile", argv[i])) {
			if (++i >= argc)
				usage(argv[i]);
			int d = atoi(argv[i]);
			profile_use(d);
			continue;
		}
		if (!strcmp ("-i", argv[i]) || !strcmp ("--init", argv[i])) {
			if (++i >= argc)
				usage(argv[i]);
			phoneConfig.initATCmd	= argv[i];
			continue;
		}
	}
	printf("Try device: %s\n", phoneConfig.devName);
	printf("Checking sound\n");
	if (openSound()) {
		closeSound();
	}
	else {
#if 0
		dialog = gtk_message_dialog_new(GTK_WINDOW(MainWindow),
						GTK_DIALOG_DESTROY_WITH_PARENT,
						GTK_MESSAGE_INFO,
						GTK_BUTTONS_CLOSE,
						"Sound output not available");
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
#endif
	}			//if (openSound())...else

#if 0
	grab_key("XF86Phone");
#endif
	db_open();
	dbus_init();
	LastUpdateTime = time(NULL) - 1;
	suspendAction.sa_handler = &suspendHandler;
	sigemptyset(&(suspendAction.sa_mask));
	suspendAction.sa_flags = SA_RESTART;
	sigaction(SIGUSR1, &suspendAction, NULL);

	resumeAction.sa_handler = &resumeHandler;
	sigemptyset(&(resumeAction.sa_mask));
	resumeAction.sa_flags = SA_RESTART;
	sigaction(SIGUSR2, &resumeAction, NULL);

	//  printf("initialising phone\n");
	// if ((returnValue = initPhone()) == 0) 
//{
	printf("Getting widget pointers\n");
	MainWindow = create_MainWindow();
//    Config = create_config();
	SignalImage = lookup_widget(GTK_WIDGET(MainWindow), "SignalImage");
	BatteryImage =
	    lookup_widget(GTK_WIDGET(MainWindow), "BatteryImage");
	DialAnswerButton =
	    lookup_widget(GTK_WIDGET(MainWindow), "DialAnswerButton");
	HangupIgnoreButton =
	    lookup_widget(GTK_WIDGET(MainWindow), "HangupIgnoreButton");
	BackspaceButton =
	    lookup_widget(GTK_WIDGET(MainWindow), "BackspaceButton");
	StatusLabel = lookup_widget(GTK_WIDGET(MainWindow), "StatusLabel");
	MissedLabel = lookup_widget(GTK_WIDGET(MainWindow), "MissedLabel");
	MsgsLabel = lookup_widget(GTK_WIDGET(MainWindow), "MsgsLabel");
	KeypadTable = lookup_widget(GTK_WIDGET(MainWindow), "KeypadTable");
	TimeLabel = lookup_widget(GTK_WIDGET(MainWindow), "TimeLabel");
	OperatorEntry = lookup_widget(GTK_WIDGET(MainWindow), "OperatorEntry");
	DialEntry = lookup_widget(GTK_WIDGET(MainWindow), "DialEntry");
	NameEntry = lookup_widget(GTK_WIDGET(MainWindow), "NameEntry");
	gtk_entry_set_alignment(GTK_ENTRY(OperatorEntry), 0.5);
	gtk_entry_set_alignment(GTK_ENTRY(DialEntry), 0.5);
	gtk_entry_set_alignment(GTK_ENTRY(NameEntry), 0.5);
	g_signal_connect ((gpointer) MainWindow, "key-press-event", (GCallback)mainwindow_key_pressed, NULL);
	gtk_widget_show(MainWindow);
	LastUpdateTime = time(NULL) + 1;
	printf("Adding idle function\n");
	printf("Initialising phone\n");
	if ((returnValue = initPhone()) == 0)
		checkSetup();
	phone_io_setup();
	audio_off();
	vibra_off();
	mixer_open();
	mixer_phone_volume_set(33);
	gtk_main();
	gtk_widget_destroy(MainWindow);
	closePhone();
	unlink(PID_FILENAME);
	return returnValue;
}				//main
