#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <X11/XF86keysym.h>
#include "support.h"
#include "interface.h"
#include "db.h"
#include "smslist.h"
#include "pdu.h"
#include "utils.h"

GtkWidget *smslist_widget;

enum {
	COL_TIME = 0,
	COL_NUMBER,
	COL_TEXT,
	NUM_COLS
};

void smslist_unrealize(GtkWidget *widget, gpointer user_data)
{
	smslist_widget=NULL;
}


static GtkTreeModel *create_and_fill_model(char *folder)
{
	GtkListStore *liststore;
	GtkTreeIter toplevel;
	GSList *res, *iter;
	char *duration;
	char *time_str;
	int hours, minutes, seconds;
	time_t now=time(NULL);
	struct pdu *pdu;


	liststore = gtk_list_store_new(NUM_COLS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
	printf("exec db_call_query\n");
	res=db_sms_query(folder);
	printf("exec db_call_query res=%p\n",res);

	
	for (iter = res; iter; iter = iter->next)
	{
		struct sms_entry *e = iter->data;
		char *number="";
		gtk_list_store_append(liststore, &toplevel);
		time_str=format_date(&now, &e->time);
		printf("decoding '%s'\n", e->pdu_data);
		pdu=pdu_decode(e->pdu_data);
		printf("decoding res %p\n", pdu);
		printf("pdu->number %p\n", pdu->number);
		if (pdu->number && pdu->number->number)
			number=pdu->number->number;
		printf("number='%s'\n", number);
		gtk_list_store_set(liststore, &toplevel, COL_TIME, time_str, COL_NUMBER, number, COL_TEXT, pdu->text,  -1);
		pdu_free(pdu);
		g_free(time_str);
		db_sms_entry_free(e);
	}
	g_slist_free(res);

		
	/* Append a top level row and leave it empty */

	printf("return\n");
	return GTK_TREE_MODEL(liststore);
}

static void create_view_and_model(GtkWidget *view, char *folder, int new)
{
	GtkTreeModel *model;
	GtkTreePath *path;

	printf("create_view_and_model\n");
	if (new) {
		gtk_tree_view_column_new_text(view, "Time", COL_TIME);
		gtk_tree_view_column_new_text(view, "Number", COL_NUMBER);
		gtk_tree_view_column_new_text(view, "Text", COL_TEXT);
	}

	model = create_and_fill_model(folder);

	gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);

	g_object_unref(model);	/* destroy model automatically with view */


	gtk_tree_selection_set_mode(gtk_tree_view_get_selection (GTK_TREE_VIEW(view)), GTK_SELECTION_SINGLE);
	path=gtk_tree_path_new_from_indices(0,-1);
	gtk_tree_view_set_cursor(GTK_TREE_VIEW(view), path, NULL, FALSE);		
}

void smslist_open(char *folder)
{
	GtkWidget *inbox;
	int new=0;

	printf("smslist_open\n");
	if (!smslist_widget) {
		smslist_widget = create_SMSList();
		g_signal_connect ((gpointer) smslist_widget, "unrealize", smslist_unrealize, NULL);
		new=1;
	} else
		window_raise(smslist_widget);
	inbox = lookup_widget(smslist_widget, "inbox_list");
	create_view_and_model(inbox,"Inbox",new);
	gtk_widget_show(smslist_widget);

}
