#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <sys/soundcard.h>

#include <stdio.h>
#include <unistd.h>
//#include <string.h>
//#include <time.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <poll.h>

#include "sound.h"

static unsigned char buffers[2][BUF_SIZE];
static FILE *sampleFile = NULL;
static int soundFile = 0, playingBuffer = -1;
static GIOChannel *iochan;



gboolean openSound(void)
{
	int speed = 16000, channels = 2, format = AFMT_S16_LE;

	if ((soundFile = open("/dev/dsp", O_WRONLY)) <= 0)
		return FALSE;

	if (ioctl(soundFile, SNDCTL_DSP_SPEED, &speed) < 0) {
		close(soundFile);
		return FALSE;
	}
	if (ioctl(soundFile, SNDCTL_DSP_CHANNELS, &channels) < 0) {
		close(soundFile);
		return FALSE;
	}
	if (ioctl(soundFile, SNDCTL_DSP_SETFMT, &format) < 0) {
		close(soundFile);
		return FALSE;
	}

	return TRUE;
}				//openSound

static void sound_read_block(int block)
{
	size_t bufIndex = 0;
	do {
		bufIndex += fread(buffers[block] + bufIndex, 1, BUF_SIZE - bufIndex, sampleFile);
		if (bufIndex < BUF_SIZE) 
			rewind(sampleFile);
	} while (bufIndex < BUF_SIZE);
}

static void sound_play_block(int block)
{
	write(soundFile, buffers[block], BUF_SIZE);
}

static gboolean updateSound(GIOChannel *source, GIOCondition condition, gpointer data)
{
	if (soundFile == 0 || sampleFile == NULL || playingBuffer == -1)
		return FALSE;
	sound_play_block(playingBuffer ^= 1);
	sound_read_block(playingBuffer ^ 1);
	return TRUE;
}


void startSound(const char *filename)
{
	printf("startSound(%s)\n", filename);

	if (playingBuffer >= 0)
		stopSound();

	if (soundFile <= 0 && !openSound())
		return;

	if ((sampleFile = fopen(filename, "rb")) == NULL)
		return;


	sound_read_block(0);
	sound_read_block(1);	
	playingBuffer = 0;
	sound_play_block(playingBuffer);
	iochan=g_io_channel_unix_new(soundFile);
        g_io_add_watch(iochan, G_IO_OUT|G_IO_ERR|G_IO_HUP, updateSound, NULL);
}



void stopSound(void)
{
	printf("stopSound\n");
	if (soundFile == 0 || sampleFile == NULL)
		playingBuffer = -1;
	if (sampleFile != NULL) {
		fclose(sampleFile);
		sampleFile = NULL;
	}
	closeSound();
}


void closeSound(void)
{
	if (playingBuffer >= 0)
		stopSound();
	if (soundFile > 0) {
		close(soundFile);
		soundFile = 0;
		if (iochan)
			g_io_channel_shutdown(iochan, FALSE, NULL);
	}
}
